@@ -29,9 +29,9 @@
 #include "buddyeditor_tool.h"
 #include "buddyeditor.h"
 
-#include <QtDesigner/QDesignerFormWindowInterface>
+#include <QtDesigner/abstractformwindow.h>
 
-#include <QtWidgets/QAction>
+#include <QtWidgets/qaction.h>
 
 QT_BEGIN_NAMESPACE
 
@@ -44,9 +44,7 @@ BuddyEditorTool::BuddyEditorTool(QDesignerFormWindowInterface *formWindow, QObje
 {
 }
 
-BuddyEditorTool::~BuddyEditorTool()
-{
-}
+BuddyEditorTool::~BuddyEditorTool() = default;
 
 QDesignerFormEditorInterface *BuddyEditorTool::core() const
 {
@@ -70,8 +68,8 @@ bool BuddyEditorTool::handleEvent(QWidget *widget, QWidget *managedWidget, QEven
 QWidget *BuddyEditorTool::editor() const
 {
     if (!m_editor) {
-        Q_ASSERT(formWindow() != 0);
-        m_editor = new BuddyEditor(formWindow(), 0);
+        Q_ASSERT(formWindow() != nullptr);
+        m_editor = new BuddyEditor(formWindow(), nullptr);
         connect(formWindow(), &QDesignerFormWindowInterface::mainContainerChanged,
                 m_editor.data(), &BuddyEditor::setBackground);
         connect(formWindow(), &QDesignerFormWindowInterface::changed,
