@@ -56,7 +56,7 @@ QDesignerPromotionInterface::~QDesignerPromotionInterface()
 */
 
 /*!
-    \fn virtual QDesignerPromotionInterface::PromotedClasses promotedClasses()  const
+    \fn QDesignerPromotionInterface::PromotedClasses QDesignerPromotionInterface::promotedClasses() const
 
     Returns a list of promoted classes along with their base classes in alphabetical order.
     It can be used to populate tree models for editing promoted widgets.
