@@ -34,6 +34,7 @@
 #include "stylesheeteditor_p.h"
 #include "qlayout_widget_p.h"
 #include "layout_p.h"
+#include "selectsignaldialog_p.h"
 #include "spacer_widget_p.h"
 #include "textpropertyeditor_p.h"
 #include "promotiontaskmenu_p.h"
@@ -45,33 +46,32 @@
 #include "qdesigner_objectinspector_p.h"
 #include "morphmenu_p.h"
 #include "formlayoutmenu_p.h"
-#include "ui_selectsignaldialog.h"
 #include "widgetfactory_p.h"
 #include "abstractintrospection_p.h"
 #include "widgetdatabase_p.h"
 
 #include <shared_enums_p.h>
 
-#include <QtDesigner/QDesignerFormWindowInterface>
-#include <QtDesigner/QDesignerFormWindowCursorInterface>
-#include <QtDesigner/QDesignerPropertySheetExtension>
-#include <QtDesigner/QDesignerFormEditorInterface>
-#include <QtDesigner/QDesignerLanguageExtension>
-#include <QtDesigner/QDesignerIntegrationInterface>
-#include <QtDesigner/QExtensionManager>
+#include <QtDesigner/abstractformwindow.h>
+#include <QtDesigner/abstractformwindowcursor.h>
+#include <QtDesigner/propertysheet.h>
+#include <QtDesigner/abstractformeditor.h>
+#include <QtDesigner/abstractlanguage.h>
+#include <QtDesigner/abstractintegration.h>
+#include <QtDesigner/qextensionmanager.h>
 
-#include <QtWidgets/QAction>
-#include <QtWidgets/QActionGroup>
-#include <QtWidgets/QWidget>
-#include <QtWidgets/QMenuBar>
-#include <QtWidgets/QMainWindow>
-#include <QtWidgets/QStatusBar>
-#include <QtWidgets/QDialogButtonBox>
-#include <QtWidgets/QVBoxLayout>
-#include <QtWidgets/QPushButton>
-#include <QtWidgets/QUndoStack>
-#include <QtCore/QDebug>
-#include <QtCore/QCoreApplication>
+#include <QtWidgets/qaction.h>
+#include <QtWidgets/qactiongroup.h>
+#include <QtWidgets/qwidget.h>
+#include <QtWidgets/qmenubar.h>
+#include <QtWidgets/qmainwindow.h>
+#include <QtWidgets/qstatusbar.h>
+#include <QtWidgets/qdialogbuttonbox.h>
+#include <QtWidgets/qboxlayout.h>
+#include <QtWidgets/qpushbutton.h>
+#include <QtWidgets/qundostack.h>
+#include <QtCore/qdebug.h>
+#include <QtCore/qcoreapplication.h>
 
 QT_BEGIN_NAMESPACE
 
@@ -84,7 +84,7 @@ static inline QAction *createSeparatorHelper(QObject *parent) {
 static QString objName(const QDesignerFormEditorInterface *core, QObject *object) {
     QDesignerPropertySheetExtension *sheet
             = qt_extension<QDesignerPropertySheetExtension*>(core->extensionManager(), object);
-    Q_ASSERT(sheet != 0);
+    Q_ASSERT(sheet != nullptr);
 
     const QString objectNameProperty = QStringLiteral("objectName");
     const int index = sheet->indexOf(objectNameProperty);
@@ -170,7 +170,7 @@ private:
 
 QAction *LayoutAlignmentMenu::createAction(const QString &text, int data, QMenu *menu, QActionGroup *ag)
 {
-    QAction * a = new QAction(text, 0);
+    QAction * a = new QAction(text, nullptr);
     a->setCheckable(true);
     a->setData(QVariant(data));
     menu->addAction(a);
@@ -250,7 +250,7 @@ bool LayoutAlignmentMenu::setAlignment(const QDesignerFormEditorInterface *core,
 
 Qt::Alignment LayoutAlignmentMenu::alignment() const
 {
-    Qt::Alignment alignment = 0;
+    Qt::Alignment alignment;
     if (const QAction *horizAction = m_horizGroup->checkedAction())
         if (const int horizAlign = horizAction->data().toInt())
             alignment |= static_cast<Qt::Alignment>(horizAlign);
@@ -283,6 +283,7 @@ public:
 
     QAction *m_addMenuBar;
     QAction *m_addToolBar;
+    QAction *m_addAreaSubMenu;
     QAction *m_addStatusBar;
     QAction *m_removeStatusBar;
     QAction *m_containerFakeMethods;
@@ -294,7 +295,7 @@ public:
 };
 
 QDesignerTaskMenuPrivate::QDesignerTaskMenuPrivate(QWidget *widget, QObject *parent) :
-    m_q(0),
+    m_q(nullptr),
     m_widget(widget),
     m_separator(createSeparatorHelper(parent)),
     m_separator2(createSeparatorHelper(parent)),
@@ -311,6 +312,7 @@ QDesignerTaskMenuPrivate::QDesignerTaskMenuPrivate(QWidget *widget, QObject *par
     m_formLayoutMenu(new FormLayoutMenu(parent)),
     m_addMenuBar(new QAction(QDesignerTaskMenu::tr("Create Menu Bar"), parent)),
     m_addToolBar(new QAction(QDesignerTaskMenu::tr("Add Tool Bar"), parent)),
+    m_addAreaSubMenu(new QAction(QDesignerTaskMenu::tr("Add Tool Bar to Other Area"), parent)),
     m_addStatusBar(new QAction(QDesignerTaskMenu::tr("Create Status Bar"), parent)),
     m_removeStatusBar(new QAction(QDesignerTaskMenu::tr("Remove Status Bar"), parent)),
     m_containerFakeMethods(new QAction(QDesignerTaskMenu::tr("Change signals/slots..."), parent)),
@@ -362,7 +364,16 @@ QDesignerTaskMenu::QDesignerTaskMenu(QWidget *widget, QObject *parent) :
     connect(d->m_changeWhatsThis, &QAction::triggered, this, &QDesignerTaskMenu::changeWhatsThis);
     connect(d->m_changeStyleSheet, &QAction::triggered, this, &QDesignerTaskMenu::changeStyleSheet);
     connect(d->m_addMenuBar, &QAction::triggered, this, &QDesignerTaskMenu::createMenuBar);
-    connect(d->m_addToolBar, &QAction::triggered, this, &QDesignerTaskMenu::addToolBar);
+    connect(d->m_addToolBar, &QAction::triggered, this,
+            [this] () { this->addToolBar(Qt::TopToolBarArea); });
+    auto areaMenu = new QMenu;
+    d->m_addAreaSubMenu->setMenu(areaMenu);
+    areaMenu->addAction(QDesignerTaskMenu::tr("Left"),
+                        [this] () { this->addToolBar(Qt::LeftToolBarArea); });
+    areaMenu->addAction(QDesignerTaskMenu::tr("Right"),
+                        [this] () { this->addToolBar(Qt::RightToolBarArea); });
+    areaMenu->addAction(QDesignerTaskMenu::tr("Bottom"),
+                        [this] () { this->addToolBar(Qt::BottomToolBarArea); });
     connect(d->m_addStatusBar, &QAction::triggered, this, &QDesignerTaskMenu::createStatusBar);
     connect(d->m_removeStatusBar, &QAction::triggered, this, &QDesignerTaskMenu::removeStatusBar);
     connect(d->m_containerFakeMethods, &QAction::triggered, this, &QDesignerTaskMenu::containerFakeMethods);
@@ -389,7 +400,7 @@ QWidget *QDesignerTaskMenu::widget() const
 QDesignerFormWindowInterface *QDesignerTaskMenu::formWindow() const
 {
     QDesignerFormWindowInterface *result = QDesignerFormWindowInterface::findFormWindow(widget());
-    Q_ASSERT(result != 0);
+    Q_ASSERT(result != nullptr);
     return result;
 }
 
@@ -408,7 +419,7 @@ void QDesignerTaskMenu::createMenuBar()
     }
 }
 
-void QDesignerTaskMenu::addToolBar()
+void QDesignerTaskMenu::addToolBar(Qt::ToolBarArea area)
 {
     if (QDesignerFormWindowInterface *fw = formWindow()) {
         QMainWindow *mw = qobject_cast<QMainWindow*>(fw->mainContainer());
@@ -418,7 +429,7 @@ void QDesignerTaskMenu::addToolBar()
         }
 
         AddToolBarCommand *cmd = new AddToolBarCommand(fw);
-        cmd->init(mw);
+        cmd->init(mw, area);
         fw->commandHistory()->push(cmd);
     }
 }
@@ -468,6 +479,7 @@ QList<QAction*> QDesignerTaskMenu::taskActions() const
                 actions.append(d->m_addMenuBar);
 
             actions.append(d->m_addToolBar);
+            actions.append(d->m_addAreaSubMenu);
             // ### create the status bar
             if (mw->findChild<QStatusBar *>(QString(), Qt::FindDirectChildrenOnly))
                 actions.append(d->m_removeStatusBar);
@@ -509,7 +521,7 @@ QList<QAction*> QDesignerTaskMenu::taskActions() const
 void QDesignerTaskMenu::changeObjectName()
 {
     QDesignerFormWindowInterface *fw = formWindow();
-    Q_ASSERT(fw != 0);
+    Q_ASSERT(fw != nullptr);
 
     const QString oldObjectName = objName(fw->core(), widget());
 
@@ -530,7 +542,7 @@ void QDesignerTaskMenu::changeTextProperty(const QString &propertyName, const QS
     QDesignerFormWindowInterface *fw = formWindow();
     if (!fw)
         return;
-    Q_ASSERT(d->m_widget->parentWidget() != 0);
+    Q_ASSERT(d->m_widget->parentWidget() != nullptr);
 
     const QDesignerPropertySheetExtension *sheet = qt_extension<QDesignerPropertySheetExtension*>(fw->core()->extensionManager(), d->m_widget);
     const int index = sheet->indexOf(propertyName);
@@ -600,22 +612,6 @@ void QDesignerTaskMenu::containerFakeMethods()
     SignalSlotDialog::editMetaDataBase(fw, d->m_widget, fw);
 }
 
-static QString declaredInClass(const QDesignerMetaObjectInterface *metaObject, const QString &member)
-{
-    // Find class whose superclass does not contain the method.
-    const QDesignerMetaObjectInterface *meta = metaObject;
-
-    for (;;) {
-        const QDesignerMetaObjectInterface *tmpMeta = meta->superClass();
-        if (tmpMeta == 0)
-            break;
-        if (tmpMeta->indexOfMethod(member) == -1)
-            break;
-        meta = tmpMeta;
-    }
-    return meta->className();
-}
-
 bool QDesignerTaskMenu::isSlotNavigationEnabled(const QDesignerFormEditorInterface *core)
 {
     return core->integration()->hasFeature(QDesignerIntegration::SlotNavigationFeature);
@@ -632,93 +628,17 @@ void QDesignerTaskMenu::navigateToSlot(QDesignerFormEditorInterface *core,
                                        QObject *object,
                                        const QString &defaultSignal)
 {
-    const QString objectName = objName(core, object);
-    QMap<QString, QMap<QString, QStringList> > classToSignalList;
-
-    // "real" signals
-    if (const QDesignerMetaObjectInterface *metaObject = core->introspection()->metaObject(object)) {
-        const int methodCount = metaObject->methodCount();
-        for (int i = 0; i < methodCount; ++i) {
-            const QDesignerMetaMethodInterface *metaMethod = metaObject->method(i);
-            if (metaMethod->methodType() == QDesignerMetaMethodInterface::Signal) {
-                const QString signature = metaMethod->signature();
-                const QStringList parameterNames = metaMethod->parameterNames();
-                classToSignalList[declaredInClass(metaObject, signature)][signature] = parameterNames;
-            }
-        }
-    }
-
-    // fake signals
-    if (qdesigner_internal::MetaDataBase *metaDataBase
-        = qobject_cast<qdesigner_internal::MetaDataBase *>(core->metaDataBase())) {
-        qdesigner_internal::MetaDataBaseItem *item = metaDataBase->metaDataBaseItem(object);
-        Q_ASSERT(item);
-        const QStringList fakeSignals = item->fakeSignals();
-        for (const QString &fakeSignal : fakeSignals)
-            classToSignalList[item->customClassName()][fakeSignal] = QStringList();
-    }
-
-    if (object->isWidgetType()) {
-        QWidget *widget = static_cast<QWidget *>(object);
-        if (WidgetDataBase *db = qobject_cast<WidgetDataBase *>(core->widgetDataBase())) {
-            const QString promotedClassName = promotedCustomClassName(core, widget);
-            const int index = core->widgetDataBase()->indexOfClassName(promotedClassName);
-            if (index >= 0) {
-                WidgetDataBaseItem* item = static_cast<WidgetDataBaseItem*>(db->item(index));
-                const QStringList fakeSignals = item->fakeSignals();
-                for (const QString &fakeSignal : fakeSignals)
-                    classToSignalList[promotedClassName][fakeSignal] = QStringList();
-            }
-        }
-    }
-
-    Ui::SelectSignalDialog dialogUi;
-    QDialog selectSignalDialog(0, Qt::WindowTitleHint | Qt::WindowSystemMenuHint);
-    dialogUi.setupUi(&selectSignalDialog);
-
-    QMap<QString, QMap<QString, QStringList> >::const_iterator iter(classToSignalList.constBegin());
-    for (; iter != classToSignalList.constEnd(); ++iter) {
-        const QString className = iter.key();
-        QMap<QString, QStringList> signalNames = iter.value();
-
-        QMap<QString, QStringList>::const_iterator itSignal(signalNames.constBegin());
-        for (; itSignal != signalNames.constEnd(); ++itSignal) {
-            const QString signalName = itSignal.key();
-            QTreeWidgetItem *row = new QTreeWidgetItem(QStringList() << signalName << className);
-            row->setData(0, Qt::UserRole, itSignal.value());
-            dialogUi.signalList->addTopLevelItem(row);
-        }
-    }
-    if (dialogUi.signalList->topLevelItemCount() == 0) {
-        QTreeWidgetItem *row = new QTreeWidgetItem(QStringList() << tr("no signals available"));
-        dialogUi.signalList->addTopLevelItem(row);
-        dialogUi.buttonBox->button(QDialogButtonBox::Ok)->setEnabled(false);
-    } else {
-        connect(dialogUi.signalList, &QTreeWidget::itemDoubleClicked,
-                &selectSignalDialog, &QDialog::accept);
-    }
-
-    if (defaultSignal.isEmpty()) {
-        dialogUi.signalList->setCurrentItem(dialogUi.signalList->topLevelItem(0));
-    } else {
-        const QList<QTreeWidgetItem *> items = dialogUi.signalList->findItems (defaultSignal, Qt::MatchExactly, 0);
-        if (!items.empty())
-            dialogUi.signalList->setCurrentItem(items.front());
-    }
-
-    dialogUi.signalList->resizeColumnToContents(0);
-
-    if (selectSignalDialog.exec() != QDialog::Accepted)
+    SelectSignalDialog dialog;
+    dialog.populate(core, object, defaultSignal);
+    if (dialog.exec() != QDialog::Accepted)
         return;
-    const QList<QTreeWidgetItem *> &selectedItems = dialogUi.signalList->selectedItems();
-    if (selectedItems.isEmpty())
-        return;
-    const QTreeWidgetItem *selectedItem = selectedItems.constFirst();
-    const QString signalSignature = selectedItem->text(0);
-    const QStringList parameterNames = qvariant_cast<QStringList>(selectedItem->data(0, Qt::UserRole));
-
     // TODO: Check whether signal is connected to slot
-    core->integration()->emitNavigateToSlot(objectName, signalSignature, parameterNames);
+    const SelectSignalDialog::Method method = dialog.selectedMethod();
+    if (method.isValid()) {
+        core->integration()->emitNavigateToSlot(objName(core, object),
+                                                method.signature,
+                                                method.parameterNames);
+    }
 }
 
 // Add a command that takes over the value of the current geometry as
@@ -760,7 +680,7 @@ void QDesignerTaskMenu::applySize(QAction *a)
 
     const int mask = a->data().toInt();
     const int size = selection.size();
-    fw->commandHistory()->beginMacro(tr("Set size constraint on %n widget(s)", 0, size));
+    fw->commandHistory()->beginMacro(tr("Set size constraint on %n widget(s)", nullptr, size));
     for (int i = 0; i < size; i++)
         createSizeCommand(fw, selection.at(i), mask);
     fw->commandHistory()->endMacro();
