@@ -30,6 +30,7 @@
 
 #include <QtCore/QByteArray>
 #include <QtCore/QDebug>
+#include <QtCore/QRegExp>
 #include <QtCore/QTextCodec>
 #include <QtCore/QTextStream>
 
@@ -187,7 +188,6 @@ QString TSReader::readTransContents()
 
 bool TSReader::read(Translator &translator)
 {
-    STRING(byte);
     STRING(catalog);
     STRING(comment);
     STRING(context);
@@ -215,13 +215,11 @@ bool TSReader::read(Translator &translator)
     STRING(type);
     STRING(unfinished);
     STRING(userdata);
-    STRING(value);
     STRING(vanished);
     //STRING(version);
     STRING(yes);
 
     static const QString strextrans(QLatin1String("extra-"));
-    static const QString strUtf8(QLatin1String("UTF-8"));
 
     while (!atEnd()) {
         readNext();
@@ -447,7 +445,8 @@ static QString protect(const QString &str)
     QString result;
     result.reserve(str.length() * 12 / 10);
     for (int i = 0; i != str.size(); ++i) {
-        uint c = str.at(i).unicode();
+        const QChar ch = str[i];
+        uint c = ch.unicode();
         switch (c) {
         case '\"':
             result += QLatin1String("&quot;");
@@ -465,7 +464,7 @@ static QString protect(const QString &str)
             result += QLatin1String("&apos;");
             break;
         default:
-            if (c < 0x20 && c != '\r' && c != '\n' && c != '\t')
+            if ((c < 0x20 || (ch > QChar(0x7f) && ch.isSpace())) && c != '\n' && c != '\t')
                 result += numericEntity(c);
             else // this also covers surrogates
                 result += QChar(c);
@@ -487,7 +486,7 @@ static void writeExtras(QTextStream &t, const char *indent,
     }
     outs.sort();
     foreach (const QString &out, outs)
-        t << indent << out << endl;
+        t << indent << out << Qt::endl;
 }
 
 static void writeVariants(QTextStream &t, const char *indent, const QString &input)
@@ -541,7 +540,7 @@ bool saveTS(const Translator &translator, QIODevice &dev, ConversionData &cd)
         t << "</dependencies>\n";
     }
 
-    QRegExp drops(cd.dropTags().join(QLatin1String("|")));
+    QRegExp drops(cd.dropTags().join(QLatin1Char('|')));
 
     writeExtras(t, "    ", translator.extras(), drops);
 
