@@ -37,7 +37,12 @@
 #ifndef QT_NO_OPENGL
 #  include <QtGui/QOpenGLContext>
 #  include <QtGui/QOpenGLFunctions>
+#  include <QtGui/QOpenGLVersionProfile>
 #endif // QT_NO_OPENGL
+#if QT_CONFIG(vulkan)
+#  include <QtGui/QVulkanInstance>
+#  include <QtGui/QVulkanWindow>
+#endif // vulkan
 #include <QtGui/QWindow>
 #include <QtGui/QTouchDevice>
 
@@ -58,6 +63,7 @@
 #include <QtCore/QDir>
 #include <QtCore/QFileSelector>
 #include <QtCore/QDebug>
+#include <QtCore/QVersionNumber>
 
 #include <private/qsimd_p.h>
 #include <private/qguiapplication_p.h>
@@ -68,6 +74,21 @@
 #include <qpa/qplatformnativeinterface.h>
 #include <private/qhighdpiscaling_p.h>
 
+#include <QtGui/private/qrhi_p.h>
+#include <QtGui/QOffscreenSurface>
+#if QT_CONFIG(opengl)
+# include <QtGui/private/qrhigles2_p.h>
+#endif
+#if QT_CONFIG(vulkan)
+# include <QtGui/private/qrhivulkan_p.h>
+#endif
+#ifdef Q_OS_WIN
+#include <QtGui/private/qrhid3d11_p.h>
+#endif
+#if defined(Q_OS_MACOS) || defined(Q_OS_IOS)
+# include <QtGui/private/qrhimetal_p.h>
+#endif
+
 #ifdef QT_WIDGETS_LIB
 #  include <QtWidgets/QStyleFactory>
 #endif
@@ -96,7 +117,7 @@ QTextStream &operator<<(QTextStream &str, const QDpi &d)
 
 QTextStream &operator<<(QTextStream &str, const QRect &r)
 {
-    str << r.size() << forcesign << r.x() << r.y() << noforcesign;
+    str << r.size() << Qt::forcesign << r.x() << r.y() << Qt::noforcesign;
     return str;
 }
 
@@ -176,9 +197,32 @@ void dumpGlInfo(QTextStream &str, bool listExtensions)
             << "\nVersion: " << reinterpret_cast<const char *>(functions.glGetString(GL_VERSION))
             << "\nShading language: " << reinterpret_cast<const char *>(functions.glGetString(GL_SHADING_LANGUAGE_VERSION))
             <<  "\nFormat: " << context.format();
-
+#  ifndef QT_OPENGL_ES_2
+        GLint majorVersion;
+        functions.glGetIntegerv(GL_MAJOR_VERSION, &majorVersion);
+        GLint minorVersion;
+        functions.glGetIntegerv(GL_MINOR_VERSION, &minorVersion);
+        const QByteArray openGlVersionFunctionsName = "QOpenGLFunctions_"
+            + QByteArray::number(majorVersion) + '_' + QByteArray::number(minorVersion);
+        str << "\nProfile: None (" << openGlVersionFunctionsName << ')';
+        if (majorVersion > 3 || (majorVersion == 3 && minorVersion >= 1)) {
+            QOpenGLVersionProfile profile;
+            profile.setVersion(majorVersion, minorVersion);
+            profile.setProfile(QSurfaceFormat::CoreProfile);
+            if (QAbstractOpenGLFunctions *f = context.versionFunctions(profile)) {
+                if (f->initializeOpenGLFunctions())
+                    str << ", Core (" << openGlVersionFunctionsName << "_Core)";
+            }
+            profile.setProfile(QSurfaceFormat::CompatibilityProfile);
+            if (QAbstractOpenGLFunctions *f = context.versionFunctions(profile)) {
+                if (f->initializeOpenGLFunctions())
+                    str << ", Compatibility (" << openGlVersionFunctionsName << "_Compatibility)";
+            }
+        }
+        str << '\n';
+#  endif // !QT_OPENGL_ES_2
         if (listExtensions) {
-            QList<QByteArray> extensionList = context.extensions().toList();
+            QByteArrayList extensionList = context.extensions().values();
             std::sort(extensionList.begin(), extensionList.end());
             str << " \nFound " << extensionList.size() << " extensions:\n";
             for (const QByteArray &extension : qAsConst(extensionList))
@@ -191,6 +235,180 @@ void dumpGlInfo(QTextStream &str, bool listExtensions)
 
 #endif // !QT_NO_OPENGL
 
+#if QT_CONFIG(vulkan)
+QVersionNumber vulkanVersion(uint32_t v)
+{
+    return QVersionNumber(VK_VERSION_MAJOR(v), VK_VERSION_MINOR(v), VK_VERSION_PATCH(v));
+}
+
+void dumpVkInfo(QTextStream &str)
+{
+    QVulkanInstance inst;
+    if (inst.create()) {
+        str << "Vulkan instance available\n";
+        str << "Supported instance extensions:\n";
+        for (const QVulkanExtension &ext : inst.supportedExtensions())
+            str << "  " << ext.name << ", version " << ext.version << "\n";
+        str << "Supported layers:\n";
+        for (const QVulkanLayer &layer : inst.supportedLayers())
+            str << "  " << layer.name << ", version " << layer.version
+                << ", spec version " << layer.specVersion.toString()
+                << ", " << layer.description << "\n";
+        // Show at least the available physical devices. Anything additional
+        // needs lots of initialization, or, if done through QVulkanWindow, an
+        // exposed window. None of these are very tempting right now.
+        str << "Available physical devices:\n";
+        QVulkanWindow window;
+        window.setVulkanInstance(&inst);
+        for (const VkPhysicalDeviceProperties &props : window.availablePhysicalDevices()) {
+            str << "  API version " << vulkanVersion(props.apiVersion).toString()
+                << Qt::hex << ", vendor 0x" << props.vendorID
+                << ", device 0x" << props.deviceID << ", " << props.deviceName
+                << Qt::dec << ", type " << props.deviceType
+                << ", driver version " << vulkanVersion(props.driverVersion).toString();
+        }
+    } else {
+        str << "Unable to create a Vulkan instance, error code is" << inst.errorCode() << "\n";
+    }
+}
+#endif // vulkan
+
+void dumpRhiBackendInfo(QTextStream &str, const char *name, QRhi::Implementation impl, QRhiInitParams *initParams)
+{
+    struct RhiFeature {
+        const char *name;
+        QRhi::Feature val;
+    };
+    const RhiFeature features[] = {
+        { "MultisampleTexture", QRhi::MultisampleTexture },
+        { "MultisampleRenderBuffer", QRhi::MultisampleRenderBuffer },
+        { "DebugMarkers", QRhi::DebugMarkers },
+        { "Timestamps", QRhi::Timestamps },
+        { "Instancing", QRhi::Instancing },
+        { "CustomInstanceStepRate", QRhi::CustomInstanceStepRate },
+        { "PrimitiveRestart", QRhi::PrimitiveRestart },
+        { "NonDynamicUniformBuffers", QRhi::NonDynamicUniformBuffers },
+        { "NonFourAlignedEffectiveIndexBufferOffset", QRhi::NonFourAlignedEffectiveIndexBufferOffset },
+        { "NPOTTextureRepeat", QRhi::NPOTTextureRepeat },
+        { "RedOrAlpha8IsRed", QRhi::RedOrAlpha8IsRed },
+        { "ElementIndexUint", QRhi::ElementIndexUint },
+        { "Compute", QRhi::Compute },
+        { "WideLines", QRhi::WideLines },
+        { "VertexShaderPointSize", QRhi::VertexShaderPointSize },
+        { "BaseVertex", QRhi::BaseVertex },
+        { "BaseInstance", QRhi::BaseInstance },
+        { "TriangleFanTopology", QRhi::TriangleFanTopology },
+        { "ReadBackNonUniformBuffer", QRhi::ReadBackNonUniformBuffer },
+        { "ReadBackNonBaseMipLevel", QRhi::ReadBackNonBaseMipLevel },
+        { nullptr, QRhi::Feature(0) }
+    };
+    struct RhiTextureFormat {
+        const char *name;
+        QRhiTexture::Format val;
+    };
+    const RhiTextureFormat textureFormats[] = {
+        { "RGBA8", QRhiTexture::RGBA8 },
+        { "BGRA8", QRhiTexture::BGRA8 },
+        { "R8", QRhiTexture::R8 },
+        { "R16", QRhiTexture::R16 },
+        { "RED_OR_ALPHA8", QRhiTexture::RED_OR_ALPHA8 },
+        { "RGBA16F", QRhiTexture::RGBA16F },
+        { "RGBA32F", QRhiTexture::RGBA32F },
+        { "R16F", QRhiTexture::R16F },
+        { "R32F", QRhiTexture::R32F },
+        { "D16", QRhiTexture::D16 },
+        { "D32F", QRhiTexture::D32F },
+        { "BC1", QRhiTexture::BC1 },
+        { "BC2", QRhiTexture::BC2 },
+        { "BC3", QRhiTexture::BC3 },
+        { "BC4", QRhiTexture::BC4 },
+        { "BC5", QRhiTexture::BC5 },
+        { "BC6H", QRhiTexture::BC6H },
+        { "BC7", QRhiTexture::BC7 },
+        { "ETC2_RGB8", QRhiTexture::ETC2_RGB8 },
+        { "ETC2_RGB8A1", QRhiTexture::ETC2_RGB8A1 },
+        { "ETC2_RGBA8", QRhiTexture::ETC2_RGBA8 },
+        { "ASTC_4x4", QRhiTexture::ASTC_4x4 },
+        { "ASTC_5x4", QRhiTexture::ASTC_5x4 },
+        { "ASTC_5x5", QRhiTexture::ASTC_5x5 },
+        { "ASTC_6x5", QRhiTexture::ASTC_6x5 },
+        { "ASTC_6x6", QRhiTexture::ASTC_6x6 },
+        { "ASTC_8x5", QRhiTexture::ASTC_8x5 },
+        { "ASTC_8x6", QRhiTexture::ASTC_8x6 },
+        { "ASTC_8x8", QRhiTexture::ASTC_8x8 },
+        { "ASTC_10x5", QRhiTexture::ASTC_10x5 },
+        { "ASTC_10x6", QRhiTexture::ASTC_10x6 },
+        { "ASTC_10x8", QRhiTexture::ASTC_10x8 },
+        { "ASTC_10x10", QRhiTexture::ASTC_10x10 },
+        { "ASTC_12x10", QRhiTexture::ASTC_12x10 },
+        { "ASTC_12x12", QRhiTexture::ASTC_12x12 },
+        { nullptr, QRhiTexture::UnknownFormat }
+    };
+
+    QScopedPointer<QRhi> rhi(QRhi::create(impl, initParams, QRhi::Flags(), nullptr));
+    if (rhi) {
+        str << name << ":\n";
+        str << "  Min Texture Size: " << rhi->resourceLimit(QRhi::TextureSizeMin) << "\n";
+        str << "  Max Texture Size: " << rhi->resourceLimit(QRhi::TextureSizeMax) << "\n";
+        str << "  Max Color Attachments: " << rhi->resourceLimit(QRhi::MaxColorAttachments) << "\n";
+        str << "  Frames in Flight: " << rhi->resourceLimit(QRhi::FramesInFlight) << "\n";
+        str << "  Uniform Buffer Alignment: " << rhi->ubufAlignment() << "\n";
+        QByteArrayList supportedSampleCounts;
+        for (int s : rhi->supportedSampleCounts())
+            supportedSampleCounts << QByteArray::number(s);
+        str << "  Supported MSAA sample counts: " << supportedSampleCounts.join(',') << "\n";
+        str << "  Features:\n";
+        for (int i = 0; features[i].name; i++) {
+            str << "    " << (rhi->isFeatureSupported(features[i].val) ? "v" : "-") << " " << features[i].name << "\n";
+        }
+        str << "  Texture formats:";
+        for (int i = 0; textureFormats[i].name; i++) {
+            if (rhi->isTextureFormatSupported(textureFormats[i].val))
+                str << " " << textureFormats[i].name;
+        }
+        str << "\n";
+    }
+}
+
+void dumpRhiInfo(QTextStream &str)
+{
+    str << "Qt Rendering Hardware Interface supported backends:\n";
+
+#if QT_CONFIG(opengl)
+    {
+        QRhiGles2InitParams params;
+        params.fallbackSurface = QRhiGles2InitParams::newFallbackSurface();
+        dumpRhiBackendInfo(str, "OpenGL (with default QSurfaceFormat)", QRhi::OpenGLES2, &params);
+        delete params.fallbackSurface;
+    }
+#endif
+
+#if QT_CONFIG(vulkan)
+    {
+        QVulkanInstance vulkanInstance;
+        vulkanInstance.create();
+        QRhiVulkanInitParams params;
+        params.inst = &vulkanInstance;
+        dumpRhiBackendInfo(str, "Vulkan", QRhi::Vulkan, &params);
+        vulkanInstance.destroy();
+    }
+#endif
+
+#ifdef Q_OS_WIN
+    {
+        QRhiD3D11InitParams params;
+        dumpRhiBackendInfo(str, "Direct3D 11", QRhi::D3D11, &params);
+    }
+#endif
+
+#if defined(Q_OS_MACOS) || defined(Q_OS_IOS)
+    {
+        QRhiMetalInitParams params;
+        dumpRhiBackendInfo(str, "Metal", QRhi::Metal, &params);
+    }
+#endif
+}
+
 #define DUMP_CAPABILITY(str, integration, capability) \
     if (platformIntegration->hasCapability(QPlatformIntegration::capability)) \
         str << ' ' << #capability;
@@ -351,7 +569,7 @@ QString qtDiag(unsigned flags)
 #endif
     str << '\n';
 
-#ifndef QT_NO_PROCESS
+#if QT_CONFIG(process)
     const QProcessEnvironment systemEnvironment = QProcessEnvironment::systemEnvironment();
     str << "\nEnvironment:\n";
     const QStringList keys = systemEnvironment.keys();
@@ -359,7 +577,7 @@ QString qtDiag(unsigned flags)
         if (key.startsWith(QLatin1Char('Q')))
            str << "  " << key << "=\"" << systemEnvironment.value(key) << "\"\n";
     }
-#endif // !QT_NO_PROCESS
+#endif // QT_CONFIG(process)
 
     const QByteArrayList features = qtFeatures();
     if (!features.isEmpty())
@@ -413,7 +631,7 @@ QString qtDiag(unsigned flags)
 #  ifndef QT_NO_SSL
     if (QSslSocket::supportsSsl()) {
         str << "Using \"" << QSslSocket::sslLibraryVersionString() << "\", version: 0x"
-            << hex << QSslSocket::sslLibraryVersionNumber() << dec;
+            << Qt::hex << QSslSocket::sslLibraryVersionNumber() << Qt::dec;
     } else {
         str << "\nSSL is not supported.";
     }
@@ -461,7 +679,7 @@ QString qtDiag(unsigned flags)
     if (passwordMaskCharacter.unicode() >= 32 && passwordMaskCharacter.unicode() < 128)
         str << '\'' << passwordMaskCharacter << '\'';
     else
-        str << "U+" << qSetFieldWidth(4) << qSetPadChar('0') << uppercasedigits << hex << passwordMaskCharacter.unicode() << dec << qSetFieldWidth(0);
+        str << "U+" << qSetFieldWidth(4) << qSetPadChar('0') << Qt::uppercasedigits << Qt::hex << passwordMaskCharacter.unicode() << Qt::dec << qSetFieldWidth(0);
     str << '\n'
         << "  fontSmoothingGamma: " << styleHints->fontSmoothingGamma() << '\n'
         << "  useRtlExtensions: " << styleHints->useRtlExtensions() << '\n'
@@ -597,21 +815,39 @@ QString qtDiag(unsigned flags)
 #ifndef QT_NO_OPENGL
     if (flags & QtDiagGl) {
         dumpGlInfo(str, flags & QtDiagGlExtensions);
-        str << "\n\n";
+        str << "\n";
     }
 #else
-    Q_UNUSED(flags)
+    Q_UNUSED(flags);
 #endif // !QT_NO_OPENGL
 
+#if QT_CONFIG(vulkan)
+    if (flags & QtDiagVk) {
+        dumpVkInfo(str);
+        str << "\n\n";
+    }
+#endif // vulkan
+
     // On Windows, this will provide addition GPU info similar to the output of dxdiag.
     if (const QPlatformNativeInterface *ni = QGuiApplication::platformNativeInterface()) {
-        const QVariant gpuInfoV = ni->property("gpu");
-        if (gpuInfoV.type() == QVariant::Map) {
-            const QString description = gpuInfoV.toMap().value(QStringLiteral("printable")).toString();
-            if (!description.isEmpty())
-                str << "\nGPU:\n" << description;
+        const QVariant gpuInfoV = ni->property("gpuList");
+        if (gpuInfoV.type() == QVariant::List) {
+            const auto gpuList = gpuInfoV.toList();
+            for (int i = 0; i < gpuList.size(); ++i) {
+                const QString description =
+                        gpuList.at(i).toMap().value(QStringLiteral("printable")).toString();
+                if (!description.isEmpty())
+                    str << "\nGPU #" << (i + 1) << ":\n" << description << '\n';
+            }
+            str << "\n";
         }
     }
+
+    if (flags & QtDiagRhi) {
+        dumpRhiInfo(str);
+        str << "\n";
+    }
+
     return result;
 }
 
